// Copyright (C) 2005-2006 Stefan Bauer <yoltia-maintain@freenet.de>

// This file is part of Yoltia.
//
// Yoltia is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License Version 2
// as published by the Free Software Foundation.
//    
// Yoltia is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Yoltia; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

#include "smain.hpp"
#include <qmenubar.h>
#include <qfiledialog.h>
#include <qaccel.h>
#include <qapplication.h>
#include <qsettings.h>
#include <qmessagebox.h>
#include "sslider.hpp"
#include "extdlg/sneu.hpp"
#include "extdlg/sset.hpp"
#include "extdlg/slistedit.hpp"

SMain::SMain() : QVBox(0 , "main") {
  const char *icon32[] = {"32 32 3 1" , ". c None" , "a c #0000ff" , "# c #ff0000" , ".....######################....." , "......####################......" , ".......##################......." , "........################........" , ".........##############........." , "#.........############.........#" , "##.........##########.........##" , "a##.........########.........###" , "####.........######.........####" , "a####.........####.........#####" , "#a####.........##.........######" , "a######..................#######" , "#a######................########" , "a#a######..............#########" , "#a########............##########" , "a#a########..........###########" , "#a#a########.........###########" , "a#a##########.........##########" , "#a#a##########.........#########" , "a#a#a##########.........########" , "#a#a############.........#######" , "a#a#a############.........######" , "#a#a#a############.........#####" , "a#a#a##############.........####" , "#a#a#a##############.........###" , "a#a#a#a##############.........##" , "#a#a#a#a#a############.........#" , "a#a#a#a#a#a#a##########........." , "#a#a#a#a#a#a#a#a########........" , "a#a#a#a#a#a#a#a#a#a######......." , "#a#a#a#a#a#a#a#a#a#a#a####......" , "a#a#a#a#a#a#a#a#a#a#a#a#a##....."};
  setIcon(QPixmap(icon32));
  QMenuBar *mainMenu = new QMenuBar(this , "mainMenu");
  worksp = new SWorkspace(this);
  worksp->setScrollBarsEnabled(true);
  connect(worksp , SIGNAL(windowActivated(QWidget *)) , this , SLOT(setPVConnections(QWidget *)));
  connect(worksp , SIGNAL(listDropped(const QStringList)) , this , SLOT(newPVFromList(const QStringList)));
  connect(worksp , SIGNAL(fileDropped(const QString)) , this , SLOT(newPVFromFile(const QString)));
  picturesMenu = new QPopupMenu(mainMenu , "picturesMenu");
  picturesMenu->setCheckable(false);
  picturesMenu->insertItem(tr("&Open...") , this , SLOT(openDlg()));
  picturesMenu->insertItem(tr("Create &Browsing-List...") , this , SLOT(createBList()));
  picturesMenu->insertItem(tr("S&lideshow...") , this , SLOT(createSlideshow()));
  picturesMenu->insertItem(tr("&New...") , this , SLOT(neuDlg()));
  connect(picturesMenu , SIGNAL(aboutToShow()) , this , SLOT(createPicturesMenu()));
  windowsMenu = new QPopupMenu(mainMenu , "windowsMenu");
  windowsMenu->insertItem(tr("Tile &vertically") , worksp , SLOT(tile()) , 0 , 0);
  windowsMenu->insertItem(tr("Tile &horizontally") , worksp , SLOT(hTile()) , 0 , 1);
  windowsMenu->insertItem(tr("&Cascade") , worksp , SLOT(cascade()) , 0 , 2);
  windowsMenu->insertItem(tr("&Minimize all") , worksp , SLOT(minimize()) , 0 , 3);
  windowsMenu->insertItem(tr("C&lose all") , worksp , SLOT(closeAll()) , 0 , 4);
  connect(windowsMenu , SIGNAL(aboutToShow()) , this , SLOT(createWindowsMenu()));
  editMenu = new QPopupMenu(mainMenu , "editMenu");
  editMenu->setCheckable(false);
  editMenu->insertItem(tr("&Settings...") , this , SLOT(einstDlg()));
  editMenu->insertSeparator();
  connect(editMenu , SIGNAL(aboutToShow()) , this , SLOT(createEditMenu()));
  macroMenu = new QPopupMenu(mainMenu , "macroMenu");
  macroMenu->setCheckable(false);
  connect(macroMenu , SIGNAL(aboutToShow()) , this , SLOT(createMacroMenu()));
  QPopupMenu *helpmenu = new QPopupMenu(mainMenu);
  helpmenu->setCheckable(false);
  helpmenu->insertItem(tr("About &Yoltia") , this , SLOT(aboutDlg()));
  helpmenu->insertItem(tr("About &Qt") , qApp , SLOT(aboutQt()));
  mainMenu->insertItem(tr("&Pictures") , picturesMenu);
  mainMenu->insertItem(tr("&Edit") , editMenu);
  mainMenu->insertItem(tr("&Macro") , macroMenu);
  mainMenu->insertItem(tr("&Windows") , windowsMenu);
  mainMenu->insertItem(tr("&Info") , helpmenu);
  statusbar = new QStatusBar(this , "statusbar");
  statusbar->setSizeGripEnabled(false);
  browseinfoStatus = new QLabel(statusbar , "browseinfoStatus");
  bildparamStatus = new QLabel(statusbar , "bildparamStatus");
  dateiinfoStatus = new QLabel(statusbar , "dateiinfoStatus");
  zoomStatus = new QLabel(statusbar , "zoomStatus");
  MPIStatus = new QLabel(statusbar , "MPIStatus");
  markStatus = new QLabel(statusbar , "markStatus");
  statusbar->addWidget(browseinfoStatus , 1);
  statusbar->addWidget(bildparamStatus , 2);
  statusbar->addWidget(dateiinfoStatus , 2);
  statusbar->addWidget(zoomStatus , 1);
  statusbar->addWidget(MPIStatus , 3);
  statusbar->addWidget(markStatus , 3);
  QAccel *keyCatcher = new QAccel(this , "keyCatcher");
  keyCatcher->insertItem(Key_Prior , 0);	//Vorhergehendes Bild im Verzeichnis laden
  keyCatcher->insertItem(Key_Next , 1);	//Nachfolgendes Bild im Verzeichnis laden
  keyCatcher->insertItem(Key_Home , 2);	//Erstes Bild im Verzeichnis laden
  keyCatcher->insertItem(Key_End , 3);	//Letztes Bild im Verzeichnis laden
  keyCatcher->insertItem(Key_S , 4);	//Bild unter aktuellem Dateipfad mit aktuellem Format speichern
  keyCatcher->insertItem(Key_Z , 5);	//Undo
  keyCatcher->insertItem(Key_R , 6);	//Redo
  keyCatcher->insertItem(Key_C , 7);	//Kopieren
  keyCatcher->insertItem(Key_L , 8);	//Reload
  keyCatcher->insertItem(Key_Plus , 9);	//Zoom-In (+20 %)
  keyCatcher->insertItem(CTRL + Key_Plus , 10);	//Zoom-In (+1 %)
  keyCatcher->insertItem(SHIFT + Key_Plus , 11);	//Zoom-In (+50 %)
  keyCatcher->insertItem(Key_Minus , 12);	//Zoom-Out (-20 %)
  keyCatcher->insertItem(CTRL + Key_Minus , 13);	//Zoom-Out (-1 %)
  keyCatcher->insertItem(SHIFT + Key_Minus , 14);	//Zoom-Out (-50 %)
  keyCatcher->insertItem(Key_D , 15);	//Delete file
  keyCatcher->insertItem(SHIFT + Key_D , 16);	//Delete file and picture
  keyCatcher->insertItem(Key_F1 , 17);	//Set zoom mode to maxA
  keyCatcher->insertItem(Key_F2 , 18);	//Set zoom mode to maxAFZG
  keyCatcher->insertItem(Key_F3 , 19);	//Set zoom mode to maxB
  keyCatcher->insertItem(Key_F4 , 20);	//Set zoom mode to maxH
  keyCatcher->insertItem(Key_F5 , 21);	//Set zoom mode to full
  keyCatcher->insertItem(Key_E , 22);   //Execute macro
  keyCatcher->insertItem(Key_Delete , 100);	//Markierten Berech freischneiden
  keyCatcher->insertItem(Key_I , 101);	//Bild invertieren
  keyCatcher->insertItem(Key_M , 102);	//Mirror vertically
  keyCatcher->insertItem(CTRL + Key_M , 103);	//Mirror horizontally
  keyCatcher->insertItem(Key_F , 104);	//Fit to max watched size
  keyCatcher->insertItem(CTRL + Key_F , 105);	//Fit to min watched size
  keyCatcher->insertItem(CTRL + Key_Left , 106);	//Remove 1 col of pixels (right edge)
  keyCatcher->insertItem(CTRL + Key_Right , 107);	//Remove 1 col of pixels (left edge)
  keyCatcher->insertItem(CTRL + Key_Up , 108);	//Remove 1 row of pixels (lower edge)
  keyCatcher->insertItem(CTRL + Key_Down , 109);	//Remove 1 row of pixels (upper edge)
  keyCatcher->insertItem(SHIFT + Key_Left , 110);	//Remove 20 cols of pixels (right edge)
  keyCatcher->insertItem(SHIFT + Key_Right , 111);	//Remove 20 cols of pixels (left edge)
  keyCatcher->insertItem(SHIFT + Key_Up , 112);	//Remove 20 rows of pixels (lower edge)
  keyCatcher->insertItem(SHIFT + Key_Down , 113);	//Remove 20 rows of pixels (upper edge)
  keyCatcher->insertItem(Key_Left , 114);	//Remove 5 cols of pixels (right edge)
  keyCatcher->insertItem(Key_Right , 115);	//Remove 5 cols of pixels (left edge)
  keyCatcher->insertItem(Key_Up , 116);	//Remove 5 rows of pixels (lower edge)
  keyCatcher->insertItem(Key_Down , 117);	//Remove 5 rows of pixels (upper edge)
  keyCatcher->insertItem(Key_A , 118);	//Auto remove single color margin around pic
  connect(keyCatcher , SIGNAL(activated(int)) , this , SLOT(hotkeyWrapper(int)));
}

void SMain::einstDlg() {
  if (SSet(this).exec() and aktPV)
    aktPV->refreshStatus();
}

void SMain::createPicturesMenu() {
  for (uchar zv = picturesMenu->count() - 1 ; zv > 3 ; zv--)	//Alte Eintr?ge l?schen (ausser den ersten 6)
    picturesMenu->removeItemAt(zv);
  picturesMenu->setItemEnabled(picturesMenu->insertItem(tr("&Save as...")) , aktPV);
  picturesMenu->setItemEnabled(picturesMenu->insertItem(tr("&Compression preview...")) , aktPV);
  if (aktPV) {	//Irgendein PicView aktiviert?
    int id = picturesMenu->idAt(4);	//Save as
    picturesMenu->setItemParameter(id , 214);
    picturesMenu->connectItem(id , aktPV , SLOT(command(int)));
    id = picturesMenu->idAt(5); //Compression preview
    picturesMenu->setItemParameter(id , 200);
    picturesMenu->connectItem(id , aktPV , SLOT(command(int)));
  }
  QSettings set;
  if (set.readBoolEntry("/Yoltia/history/show" , true)) {	//Letztenliste erstellen?
    picturesMenu->insertSeparator();
    QStringList history(set.readListEntry("/Yoltia/history/list"));
    {	//L?schen der überzähligen Eintr?ge des Verlaufes
      const uchar historyN = set.readNumEntry("/Yoltia/history/count" , 10);
      if (history.count() > historyN)	//Maximale Listenl?nge erreicht?
	history.erase(history.at(historyN) , --history.end());
    }
    if (history.count()) {	//BUGFIX Hang on opening File-menu when no settings file exits
      QString displayText;
      for (uchar zv = 0 ; zv < history.count() - 1 ; zv++) {
        displayText = history[zv];
	if (history[zv].find("&") > -1)
	  displayText.insert(displayText.find("&") , "&");
        picturesMenu->setItemParameter(picturesMenu->insertItem(QString("&%1: %2").arg(zv).arg(displayText) , this , SLOT(ladeLetzte(int))) , zv);
      }
    }
    set.writeEntry("/Yoltia/history/list" , history);
  }
}

void SMain::createEditMenu() {
  for (uchar zv = editMenu->count() - 1 ; zv > 1 ; zv--)	//Alte Eintr?ge l?schen (ausser dem ersten) (notwendig, da sonst erst die Items aufwändig disconnected werden müssen)
    editMenu->removeItemAt(zv);
  editMenu->setItemEnabled(editMenu->insertItem(tr("&Browsing-List...")) , aktPV);
  editMenu->setItemEnabled(editMenu->insertItem(tr("&Insert...")) , aktPV);
  editMenu->setItemEnabled(editMenu->insertItem(tr("&Rotate...")) , aktPV);
  editMenu->setItemEnabled(editMenu->insertItem(tr("R&esize...")) , aktPV);
  editMenu->setItemEnabled(editMenu->insertItem(tr("&Depth conversion...")) , aktPV);
  editMenu->setItemEnabled(editMenu->insertItem(tr("R&GB adjust...")) , aktPV);
  editMenu->setItemEnabled(editMenu->insertItem(tr("&HSV adjust...")) , aktPV);
  editMenu->setItemEnabled(editMenu->insertItem(tr("Gradation &curve...")) , aktPV);
  if (aktPV) {	//Irgendein PicView aktiviert?
    int id = editMenu->idAt(2);	//List editor
    editMenu->setItemParameter(id , 201);
    editMenu->connectItem(id , aktPV , SLOT(command(int)));
    id = editMenu->idAt(3);	//Insert
    editMenu->setItemParameter(id , 202);
    editMenu->connectItem(id , aktPV , SLOT(command(int)));
    id = editMenu->idAt(4);	//Rotate
    editMenu->setItemParameter(id , 203);
    editMenu->connectItem(id , aktPV , SLOT(command(int)));
    id = editMenu->idAt(5);	//Resize
    editMenu->setItemParameter(id , 204);
    editMenu->connectItem(id , aktPV , SLOT(command(int)));
    id = editMenu->idAt(6);	//Depth conversion
    editMenu->setItemParameter(id , 205);
    editMenu->connectItem(id , aktPV , SLOT(command(int)));
    id = editMenu->idAt(7);	//RGB adjust
    editMenu->setItemParameter(id , 206);
    editMenu->connectItem(id , aktPV , SLOT(command(int)));
    id = editMenu->idAt(8);	//HSV adjust
    editMenu->setItemParameter(id , 207);
    editMenu->connectItem(id , aktPV , SLOT(command(int)));
    id = editMenu->idAt(9);	//Gradation curve
    editMenu->setItemParameter(id , 208);
    editMenu->connectItem(id , aktPV , SLOT(command(int)));
  }
}

void SMain::createMacroMenu() {
  macroMenu->clear();
  macroMenu->setItemEnabled(macroMenu->insertItem(tr("(&Re-)Start Recording")) , aktPV);
  macroMenu->setItemEnabled(macroMenu->insertItem(tr("&Stop Recording")) , aktPV);
  macroMenu->setItemEnabled(macroMenu->insertItem(tr("&Load...")) , aktPV);
  macroMenu->setItemEnabled(macroMenu->insertItem(tr("Sa&ve...")) , aktPV);
  macroMenu->setItemEnabled(macroMenu->insertItem(tr("Show &actions...")) , aktPV);
  if (aktPV) {
    int id = macroMenu->idAt(0);	//(Re-)Start Recording
    macroMenu->setItemParameter(id , 209);
    macroMenu->connectItem(id , aktPV , SLOT(command(int)));
    id = macroMenu->idAt(1);	//Stop Recording
    macroMenu->setItemParameter(id , 210);
    macroMenu->connectItem(id , aktPV , SLOT(command(int)));
    macroMenu->setItemEnabled(id , aktPV->isRecordingMacro());
    id = macroMenu->idAt(2);	//Load
    macroMenu->setItemParameter(id , 211);
    macroMenu->connectItem(id , aktPV , SLOT(command(int)));
    id = macroMenu->idAt(3);	//Save
    macroMenu->setItemParameter(id , 212);
    macroMenu->connectItem(id , aktPV , SLOT(command(int)));
    macroMenu->setItemEnabled(id , aktPV->hasMacro());
    id = macroMenu->idAt(4);	//Show actions
    macroMenu->setItemParameter(id , 213);
    macroMenu->connectItem(id , aktPV , SLOT(command(int)));
    macroMenu->setItemEnabled(id , aktPV->hasMacro());
  }
}

void SMain::newPVFromFile(const QString dateipfad) {
  SPicView *newPV = new SPicView(worksp);
  if (newPV->createWithDir(dateipfad))
      newPV->show();
    else {
      delete newPV;
      statusbar->message(tr("An error occured while loading file \"%1\".").arg(dateipfad) , LONGDELAY); //LONGDELAY defined in spicview.hpp
  }
}

void SMain::setPVConnections(QWidget *w) {
  if (aktPV)	//Vorheriges aktives PicView bekannt?
    aktPV->disconnect();
  aktPV = (SPicView *) w;
  if (aktPV) { //"Note that w can be null" (Qt Assistant)
      connect(aktPV , SIGNAL(browseinfoChanged(const QString &)) , browseinfoStatus , SLOT(setText(const QString &)));	//Browseinfo ?bergeben
      connect(aktPV , SIGNAL(bildparamChanged(const QString &)) , bildparamStatus , SLOT(setText(const QString &)));	//Bildgr??e ?bermitteln
      connect(aktPV , SIGNAL(dateiinfoChanged(const QString &)) , dateiinfoStatus , SLOT(setText(const QString &)));
      connect(aktPV , SIGNAL(zoomChanged(const QString &)) , zoomStatus , SLOT(setText(const QString &)));	//Zoom ?bermitteln
      connect(aktPV , SIGNAL(MPIChanged(const QString &)) , MPIStatus , SLOT(setText(const QString &)));
      connect(aktPV , SIGNAL(markChanged(const QString &)) , markStatus , SLOT(setText(const QString &)));
      connect(aktPV , SIGNAL(meldung(const QString & , int)) , statusbar , SLOT(message(const QString & , int)));
      aktPV->refreshStatus();
    } else {
      browseinfoStatus->setText(0);
      bildparamStatus->setText(0);
      dateiinfoStatus->setText(0);
      zoomStatus->setText(0);
      MPIStatus->setText(0);
      markStatus->setText(0);
  }
}

void SMain::ladeLetzte(int index) {
  newPVFromFile(QSettings().readListEntry("/Yoltia/history/list")[index]);
}

void SMain::createWindowsMenu() {
  for (uchar zv = windowsMenu->count() - 1 ; zv > 4 ; zv--)	//Alte Eintr?ge l?schen
    windowsMenu->removeItemAt(zv);
  for (uchar zv = 0 ; zv < 5 ; zv++)
    windowsMenu->setItemEnabled(zv , aktPV);
  QWidgetList fenster = worksp->windowList();
  if (fenster.count()) {
    windowsMenu->insertSeparator();
    for (int zv = 0 ; zv < int(fenster.count()) ; zv++)
      windowsMenu->setItemChecked(windowsMenu->insertItem("&" + QString::number(zv) + ": " + fenster.at(zv)->caption() , fenster.at(zv) , SLOT(showNormal())) , worksp->activeWindow() == fenster.at(zv));
  }
}

void SMain::closeEvent(QCloseEvent *event) {
  worksp->closeAll();
  if (!aktPV)
      event->accept();
    else
      event->ignore();
}

void SMain::neuDlg() {
  SNeu neu(this);
  if (neu.exec() == QDialog::Accepted) {
    SPicView *newPV = new SPicView(worksp);
    newPV->createWithOne(neu.sizeChanger->getWidth() , neu.sizeChanger->getHeight() , neu.getDepth() , neu.fillColor);
    newPV->show();
  }
}

void SMain::openDlg() {
  QString file = QFileDialog::getOpenFileName(QSettings().readEntry("/Yoltia/history/dir" , QDir::homeDirPath()) , tr("All image files (*.[bB][mM][pP] *.[gG][iI][fF] *.[mM][nN][gG] *.[jJ][pP][gG] *.[jJ][pP][eE][gG] *.[pP][bB][mM] *.[pP][gG][mM] *.[pP][nN][gG] *.[pP][pP][mM] *.[xX][bB][mM] *.[xX][pP][mM]);;All files (*);;BMP (*.[bB][mM][pP]);;GIF (*.[gG][iI][fF]);;MNG (*.[mM][nN][gG]);;JPEG (*.[jJ][pP][gG] *.[jJ][pP][eE][gG]);;PNG (*.[pP][nN][gG]);;PNM (*.[pP][bB][mM] *.[pP][gG][mM] *.[pP][pP][mM]);;XBM (*.[xX][bB][mM]);;XPM (*.[xX][pP][mM])") , this , "openFileDlg" , tr("Choose file"));
  if (!file.isEmpty())	//File selected (not mandatory)
    newPVFromFile(file);
}

void SMain::newPVFromList(const QStringList list) {
  SPicView *newPV = new SPicView(worksp);
  if (newPV->createWithOwn(list))
      newPV->show();
    else {
      delete newPV;
      statusbar->message(tr("Error") , LONGDELAY); //LONGDELAY defined in spicview.hpp
  }
}

void SMain::aboutDlg() {
  QMessageBox::about(this , tr("About Yoltia") , tr("%1Yoltia is licensed under the terms of the\nGNU GPL Version 2. For more information\nread the attatched file \"COPYING\".\n\nHomepage and E-Mail:%2").arg("Yoltia Version 0.21.0\n\nCopyright (C) 2005-2006 Stefan Bauer\n\n" , "\nhttp://people.freenet.de/yoltia\nyoltia-maintain@freenet.de"));
}

void SMain::createBList() {
  SListEdit listedit(this , QStringList());
  if (listedit.exec())
    newPVFromList(listedit.getList());
}

void SMain::createSlideshow() {
  SListEdit listedit(this , QStringList());
  if (listedit.exec() and !listedit.getList().isEmpty())
    new SSlider(listedit.getList());
}

